package wiki

import (
	"net/http"

	"github.com/Sirupsen/logrus"
	"github.com/dalu/wiki/wiki/storage"
	"github.com/dalu/wiki/wiki/template"
	"github.com/microcosm-cc/bluemonday"
	"github.com/russross/blackfriday"
)

const DefaultMountPath = "/wiki/"

const (
	methodGET  = "GET"
	methodPOST = "POST"
)

type wikiHandler struct {
	s storage.Interface
	r template.Interface
	l *logrus.Logger
}

func NewWikiHandler(s storage.Interface, r template.Interface, l *logrus.Logger) *wikiHandler {
	l.Level = logrus.DebugLevel
	return &wikiHandler{
		s: s,
		r: r,
		l: l,
	}
}

func (h *wikiHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	switch r.Method {
	case methodGET:
		h.handleGET(w, r)
	case methodPOST:
		h.handlePOST(w, r)
	default:
		http.Error(w, http.StatusText(http.StatusMethodNotAllowed), http.StatusMethodNotAllowed)
	}
}

func (h *wikiHandler) handleGET(w http.ResponseWriter, r *http.Request) {
	action := r.URL.Query().Get("action")

	switch action {
	case "":
		h.show(w, r)
	case "edit":
		h.l.Debug("edit hit")
	case "create":
		h.create(w, r)
	case "revision":
		h.l.Debug("revision hit")
	case "revisions":
		h.l.Debug("revisions hit")
	}
}

func (h *wikiHandler) handlePOST(w http.ResponseWriter, r *http.Request) {}

// -----

func (h *wikiHandler) show(w http.ResponseWriter, r *http.Request) {
	h.l.Debug("show hit")
	term, e := h.s.GetTermBySlug(r.URL.Path)
	if e != nil {
		h.notFound(w, r)
		return
	}
	if term.Redirect != "" {
		http.Redirect(w, r, DefaultMountPath+term.Redirect, http.StatusTemporaryRedirect)
		return
	}
	if term.CurrentRevisionID == "" {
		h.notFound(w, r)
		//http.Redirect(w, r, DefaultMountPath+r.URL.Path+"?action=new", http.StatusTemporaryRedirect)
		return
	}
	revision, e := h.s.GetRevisionByID(term.CurrentRevisionID)
	if e != nil {
		h.l.Error("wikihandler.show.GetRevisionByTermID:", e)
		return
	}

	revision.HTML = string(bluemonday.UGCPolicy().SanitizeBytes(blackfriday.MarkdownCommon([]byte(revision.Text))))

	ctx := make(map[string]interface{})
	ctx["term"] = term
	ctx["revision"] = revision

	if e := h.r.Render(w, "wiki/show", ctx); e != nil {
		h.l.Error("wikiHandler.show.Renderer.write:", e)
	}
}

func (h *wikiHandler) notFound(w http.ResponseWriter, r *http.Request) {
	h.l.Debug("not found hit")
	w.WriteHeader(http.StatusNotFound)

	ctx := make(map[string]interface{})
	ctx["term"] = unslugify(r.URL.Path)
	ctx["path"] = r.URL.Path
	if e := h.r.Render(w, "wiki/not-found", ctx); e != nil {
		h.l.Error("wikiHandler.notFound.Renderer.write:", e)
	}
}

func (h *wikiHandler) create(w http.ResponseWriter, r *http.Request) {
	h.l.Debug("create hit")

}

